@@ -18,10 +18,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
 import time
-from . import odict
-from Element import Element
+from operator import methodcaller
+from itertools import ifilter
+
 from .. gui import Messages
-from . Constants import FLOW_GRAPH_FILE_FORMAT_VERSION
+
+from . import odict
+from .Element import Element
+from .Constants import FLOW_GRAPH_FILE_FORMAT_VERSION
 
 
 class FlowGraph(Element):
@@ -63,6 +67,58 @@ class FlowGraph(Element):
     def __str__(self):
         return 'FlowGraph - %s(%s)' % (self.get_option('title'), self.get_option('id'))
 
+    def get_complexity(self):
+        """
+        Determines the complexity of a flowgraph
+        """
+        dbal = 0
+        block_list = self.get_blocks()
+        for block in block_list:
+            # Skip options block
+            if block.get_key() == 'options':
+                continue
+
+            # Don't worry about optional sinks?
+            sink_list = filter(lambda c: not c.get_optional(), block.get_sinks())
+            source_list = filter(lambda c: not c.get_optional(), block.get_sources())
+            sinks = float(len(sink_list))
+            sources = float(len(source_list))
+            base = max(min(sinks, sources), 1)
+
+            # Port ratio multiplier
+            if min(sinks, sources) > 0:
+                multi = sinks / sources
+                multi = (1 / multi) if multi > 1 else multi
+            else:
+                multi = 1
+
+            # Connection ratio multiplier
+            sink_multi = max(float(sum(map(lambda c: len(c.get_connections()), sink_list)) / max(sinks, 1.0)), 1.0)
+            source_multi = max(float(sum(map(lambda c: len(c.get_connections()), source_list)) / max(sources, 1.0)), 1.0)
+            dbal = dbal + (base * multi * sink_multi * source_multi)
+
+        elements = float(len(self.get_elements()))
+        connections = float(len(self.get_connections()))
+        disabled_connections = len(filter(lambda c: not c.get_enabled(), self.get_connections()))
+        blocks = float(len(block_list))
+        variables = elements - blocks - connections
+        enabled = float(len(self.get_enabled_blocks()))
+
+        # Disabled multiplier
+        if enabled > 0:
+            disabled_multi = 1 / (max(1 - ((blocks - enabled) / max(blocks, 1)), 0.05))
+        else:
+            disabled_multi = 1
+
+        # Connection multiplier (How many connections )
+        if (connections - disabled_connections) > 0:
+            conn_multi = 1 / (max(1 - (disabled_connections / max(connections, 1)), 0.05))
+        else:
+            conn_multi = 1
+
+        final = round(max((dbal - 1) * disabled_multi * conn_multi * connections, 0.0) / 1000000, 6)
+        return final
+
     def rewrite(self):
         def refactor_bus_structure():
 
@@ -98,7 +154,7 @@ class FlowGraph(Element):
                                 get_p().append(port);
 
         for child in self.get_children(): child.rewrite()
-        refactor_bus_structure();
+        refactor_bus_structure()
 
     def get_option(self, key):
         """
@@ -118,18 +174,24 @@ class FlowGraph(Element):
     ##############################################
     ## Access Elements
     ##############################################
-    def get_block(self, id): return filter(lambda b: b.get_id() == id, self.get_blocks())[0]
-    def get_blocks_unordered(self): return filter(lambda e: e.is_block(), self.get_elements())
+    def get_block(self, id):
+        for block in self.iter_blocks():
+            if block.get_id() == id:
+                return block
+        raise KeyError('No block with ID {0!r}'.format(id))
+
+    def iter_blocks(self):
+        return ifilter(methodcaller('is_block'), self.get_elements())
+
     def get_blocks(self):
-        # refactored the slow, ugly version
-        # don't know why we need this here, using it for sorted export_data()
-        return sorted(self.get_blocks_unordered(), key=lambda b: (
-            b.get_key() != 'options',  # options to the front
-            not b.get_key().startswith('variable'),  # then vars
-            str(b)
-        ))
-    def get_connections(self): return filter(lambda e: e.is_connection(), self.get_elements())
-    def get_children(self): return self.get_elements()
+        return list(self.iter_blocks())
+
+    def iter_connections(self):
+        return ifilter(methodcaller('is_connection'), self.get_elements())
+
+    def get_connections(self):
+        return list(self.iter_connections())
+
     def get_elements(self):
         """
         Get a list of all the elements.
@@ -145,6 +207,14 @@ class FlowGraph(Element):
             self._elements.remove(self._options_block)
         return self._elements
 
+    get_children = get_elements
+
+    def iter_enabled_blocks(self):
+        """
+        Get an iterator of all blocks that are enabled and not bypassed.
+        """
+        return ifilter(methodcaller('get_enabled'), self.iter_blocks())
+
     def get_enabled_blocks(self):
         """
         Get a list of all blocks that are enabled and not bypassed.
@@ -152,7 +222,7 @@ class FlowGraph(Element):
         Returns:
             a list of blocks
         """
-        return filter(lambda b: b.get_enabled(), self.get_blocks())
+        return list(self.iter_enabled_blocks())
 
     def get_bypassed_blocks(self):
         """
@@ -161,7 +231,7 @@ class FlowGraph(Element):
         Returns:
             a list of blocks
         """
-        return filter(lambda b: b.get_bypassed(), self.get_blocks())
+        return filter(methodcaller('get_bypassed'), self.iter_blocks())
 
     def get_enabled_connections(self):
         """
@@ -170,7 +240,7 @@ class FlowGraph(Element):
         Returns:
             a list of connections
         """
-        return filter(lambda c: c.get_enabled(), self.get_connections())
+        return filter(methodcaller('get_enabled'), self.get_connections())
 
     def get_new_block(self, key):
         """
@@ -252,10 +322,17 @@ class FlowGraph(Element):
         Returns:
             a nested data odict
         """
+        # sort blocks and connections for nicer diffs
+        blocks = sorted(self.iter_blocks(), key=lambda b: (
+            b.get_key() != 'options',  # options to the front
+            not b.get_key().startswith('variable'),  # then vars
+            str(b)
+        ))
+        connections = sorted(self.get_connections(), key=str)
         n = odict()
         n['timestamp'] = self._timestamp
-        n['block'] = [b.export_data() for b in self.get_blocks()]  # already sorted
-        n['connection'] = [c.export_data() for c in sorted(self.get_connections(), key=str)]
+        n['block'] = [b.export_data() for b in blocks]
+        n['connection'] = [c.export_data() for c in connections]
         instructions = odict({
             'created': self.get_parent().get_version_short(),
             'format': FLOW_GRAPH_FILE_FORMAT_VERSION,
@@ -297,11 +374,11 @@ class FlowGraph(Element):
                 block = self.get_new_block('dummy_block')
                 # Ugly ugly ugly
                 _initialize_dummy_block(block, block_n)
-                Messages.send_error_msg_load('Block key "%s" not found in %s' % (key, self.get_parent()))
+                Messages.send_error_msg_load('Block key "%s" not found' % key)
 
             block.import_data(block_n)
         #build the connections
-        block_ids = map(lambda b: b.get_id(), self.get_blocks())
+        block_ids = map(methodcaller('get_id'), self.iter_blocks())
         for connection_n in connections_n:
             try:  # to make the connection
                 #get the block ids
